/*********************************************************************
 *
 * File:         daemon.c
 *
 * Description:  Daemon for conference engine
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-14-96
 *
 *********************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <netdb.h>
#include <unistd.h>

#include "global.h"
#include "daemon.h"

DaemonData daemonData;

void nic()
{
}


/* The lowest level control packages sending procedure */
int sendControlPacket(struct sockaddr_in *addr, char *msg, int len)
{
  int size;
  fd_set set;
  struct timeval timeout;

  timeout.tv_sec = 1;
  FD_ZERO(&set);
  FD_SET(daemonData.socket, &set);
  if(select(daemonData.socket+1, 0, &set, 0, &timeout) > 0)
    {
      if(sendto(daemonData.socket, msg, len, 
		0, addr, sizeof(struct sockaddr_in)) < 0)
	return 0;  
    }
  return 1;
}

/* Daemon initialization */
void initDaemon()
{
  if ((daemonData.socket = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    fprintf(stderr, "Engine: can't open daemon socket\n");

  bzero(&daemonData.addr, sizeof(daemonData.addr));
  daemonData.addr.sin_family = AF_INET;
  daemonData.addr.sin_addr.s_addr = htonl(INADDR_ANY);
  daemonData.addr.sin_port        = htons(DAEMON_PORT);

  if(bind(daemonData.socket, &daemonData.addr, sizeof(daemonData.addr)) < 0)
    fprintf(stderr, "Engine: can't bind daemon socket\n");
}


/**************************************************************************
 *
 * Main socket reading procedure
 *
 **************************************************************************/
void runDaemon()
{
  struct sockaddr_in remote_addr;
  int remote_addr_len;
  int msg_len;
  int n, total;
  int tmp_socket;
  char *msg;
  fd_set set;
  struct timeval timeout;

  remote_addr_len = sizeof(remote_addr);
  timeout.tv_sec = 1;
  while(1)
    {
      FD_ZERO(&set);
      FD_SET(daemonData.socket, &set);
      if(select(daemonData.socket+1, &set, 0, 0, &timeout) > 0)
	{
	  if(FD_ISSET(daemonData.socket, &set))
	    {
	      msg = malloc(MAX_UDP_SIZE);
	      n = recvfrom(daemonData.socket, msg, MAX_UDP_SIZE, 
			   0, &remote_addr, &remote_addr_len); 
	      
	      if(n > 0)
		messageHandle(msg, n, &remote_addr);
	      else
		fprintf(stderr, "Engine: recvfrom error\n");
	      
	      free(msg);
	    }
	}
    }
}

/*********************************************************************
 *
 * Incoming messages handling procedures
 *
 *********************************************************************/
void messageHandle(char *msg, int len, struct sockaddr_in *addr)
{
  RawMessage *message;

  message = (RawMessage *)msg;
  if(message->local == 0)
    handleRemoteMessage((char *)message, len);
  else
    handleLocalMessage((char *)message, len);
}

void handleLocalMessage(char *msg, int len)
{
  Message *message;
  int size;

  message = (Message *)msg;
  size = len - sizeof(Message) + 1;

  switch((LocalTag)message->tag)
    {
    case reqLocalSessionInit:
      PRINTF("Engine: local session init\n");
      sessionInit();
      break;

    case reqLocalSessionCreate:
      PRINTF("Engine: local session create\n");
      sessionCreate(&message->sender);
      break;

    case reqLocalSessionAddMember:
      PRINTF("Engine: local session add member\n");
      localClbSessionAddMember(message, len);
      break;

    case reqLocalSessionJoin:
      PRINTF("Engine: local session join\n");
      localClbSessionJoin(message, len);
      break;

    case reqLocalSessionGetMembers:
      PRINTF("Engine: local session get mebers\n");
      localClbSessionGetMembers(message, len);
      break;

    case reqLocalSessionQuit:
      PRINTF("Engine: local session quit\n");
      localClbSessionQuit(message, len);
      break;

    case reqLocalSessionSendToApp:
      PRINTF("Engine: local session send to application\n");
      localClbSessionSendToApp(message, len);
      break;

    case reqLocalSessionStartApp:
      PRINTF("Engine: local session start application\n");
      localClbSessionStartApp(message, len);
      break;

    case reqLocalApplicationAttach:
      PRINTF("Engine: local application attach\n");
      localClbAppAttach(message, len);
      break;

    case reqLocalApplicationDettach:
      PRINTF("Engine: local application dettach\n");
      localClbAppDettach(message, len);
      break;

    default:
      fprintf(stderr, "Engine: Unknown local tag\n");
    }
}

void handleRemoteMessage(char *msg, int len)
{
  Message *message;
  int size;

  message = (Message *)msg;
  size = len - sizeof(Message) + 1;
  
  switch((RemoteTag)message->tag)
    {
    case reqRemoteSessionInvite: 
      PRINTF("Engine: remote session invited\n");
      remoteClbSessionInvite(message, len);
      break;

    case reqRemoteSessionAccept:
      PRINTF("Engine: remote session accept\n");
      remoteClbSessionAccept(message, len);
      break;

    case reqRemoteSessionReject:
      PRINTF("Engine: remote: session reject\n");
      remoteClbSessionReject(message, len);
      break;

    case reqRemoteSessionAddMember:
      PRINTF("Engine: remote session add member\n");
      remoteClbSessionAddMember(message, len);
      break;

    case reqRemoteSessionAddRest:
      PRINTF("Engine: remote session add rest\n");
      remoteClbSessionAddRest(message, len);
      break;

    case reqRemoteSessionQuit:
      PRINTF("Engine: remote session quit\n");
      remoteClbSessionQuit(message, len);
      break;

    case reqRemoteSessionSendToApp:
      PRINTF("Engine: remote session send to application\n");
      remoteClbSessionSendToApp(message, len);
      break;

    case reqRemoteSessionStartApp:
      PRINTF("Engine: remote session start application\n");
      remoteClbSessionStartApp(message, len);
      break;

    default:
      fprintf(stderr, "Engine: Unknown remote tag\n");
    }
}



/***********************************************************************
 *
 *                            MAIN
 *
 **********************************************************************/
void main(void)
{
  initDaemon();
  runDaemon();
}










